﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;
namespace DCPP
{
    public class BatarangPizza : MonoBehaviour
    {
        private SpriteRenderer LightSprite;
        private PhysicalBehaviour PhysicalBehaviour;
        private Rigidbody2D body;
        private bool Enabled;
        private float spinTime;
        private AudioSource Aud;
        private Transform Prongs;
        private Coroutine SavedCoroutine;
        private GameObject ElectricStuff;

        public enum Modes
        {
            Normal, //h
            Electric, //h
            Explosive, //h
            Controllable, //h
            Boomerang, //h
            Pizza
        };

        public Modes Mode = Modes.Normal;

        private void Awake()
        {
            this.PhysicalBehaviour = this.gameObject.GetComponent<PhysicalBehaviour>();
            this.Aud = this.gameObject.AddComponent<AudioSource>();
            this.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            this.body = this.gameObject.GetComponent<Rigidbody2D>();
            GameObject Light = new GameObject("Lights", new Type[] 
            { 
                typeof(SpriteRenderer)
            });
            Light.transform.SetParent(this.transform);
            Light.transform.SetPositionAndRotation(this.transform.position, this.transform.rotation);
            this.LightSprite = Light.GetComponent<SpriteRenderer>();
            this.LightSprite.sortingLayerID = this.gameObject.GetComponent<SpriteRenderer>().sortingLayerID;
            this.LightSprite.sortingOrder = this.gameObject.GetComponent<SpriteRenderer>().sortingOrder + 1;
            this.LightSprite.sprite = ResourceStorage.Sprite_BatarangLight;
            this.LightSprite.material = Resources.Load<GameObject>("Prefabs/ModLightPrefab").GetComponent<LightSprite>().SpriteRenderer.material;
            this.LightSprite.color = new Color(1f, 0.4f, 0f, 80f);
            //this.LightSprite.material.SetColor("_GlowColour", new Color(1f, 0.4f, 0f, 50f));
            this.LightSprite.material.SetFloat("_GlowIntensity", 0.25f);
            this.LightSprite.enabled = false;
            GameObject Prong = new GameObject("Prongs", new Type[]
            {
                typeof(SpriteRenderer)
            });
            Prong.transform.SetParent(this.transform);
            Prong.transform.SetPositionAndRotation(this.transform.position, this.transform.rotation);
            Prong.GetComponent<SpriteRenderer>().sortingLayerID = this.gameObject.GetComponent<SpriteRenderer>().sortingLayerID;
            Prong.GetComponent<SpriteRenderer>().sortingOrder = this.gameObject.GetComponent<SpriteRenderer>().sortingOrder - 1;
            Prong.GetComponent<SpriteRenderer>().sprite = ResourceStorage.Sprite_BatarangProngs;
            this.Prongs = Prong.transform;
            this.body.drag = 0f;
            this.ElectricStuff = UnityEngine.Object.Instantiate<GameObject>(ModAPI.FindSpawnable("Generator").Prefab.GetComponent<GeneratorBehaviour>().particles.gameObject);
            this.ElectricStuff.transform.SetParent(this.transform);
            this.ElectricStuff.transform.SetPositionAndRotation(this.transform.position, this.transform.rotation);
            this.ElectricStuff.transform.localScale = new Vector2(0f, 0f);
            this.ElectricStuff.transform.localPosition = new Vector2(0f, -0.16f);
            this.ElectricStuff.GetComponent<ParticleSystem>().Stop(); //This was not the intended effect :( If someone could get the generator particles to match with the prongs, thanks.
            foreach (Modes themode in Enum.GetValues(typeof(Modes)))
            {
                this.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(new ContextMenuButton("Change Mode: " + themode.ToString(), delegate ()
                {
                    return "Change Mode: " + themode.ToString();
                }, "Change Mode: " + themode.ToString(), new UnityAction[]
                {
                    delegate()
                    {
                        this.Mode = themode;
                        if (themode != Modes.Electric)
                        {
                            if (SavedCoroutine != null)
                            {
                                this.StopCoroutine(SavedCoroutine);
                            }
                            this.SavedCoroutine = this.StartCoroutine(LowerProngs());
                        }
                        switch (themode)
                        {
                            case Modes.Explosive:
                                this.LightSprite.enabled = true;
                                this.LightSprite.color = new Color(1f, 0.4f, 0f, 1f);
                                this.LightSprite.material.SetFloat("_GlowIntensity", 0.7f);
                                break;

                            case Modes.Electric:
                                this.LightSprite.enabled = true;
                                this.LightSprite.color = new Color(0f, 0.85f, 1f, 1f);
                                this.LightSprite.material.SetFloat("_GlowIntensity", 1f);
                                if (SavedCoroutine != null)
                                {
                                    this.StopCoroutine(SavedCoroutine);
                                }
                                this.SavedCoroutine = this.StartCoroutine(RaiseProngs());
                                break;

                            case Modes.Controllable:
                                this.LightSprite.enabled = true;
                                this.LightSprite.color = new Color(0f, 1f, 0.9f, 1f);
                                this.LightSprite.material.SetFloat("_GlowIntensity", 1f);
                                break;

                            default:
                                this.LightSprite.enabled = false;
                                this.LightSprite.material.SetFloat("_GlowIntensity", 0f);
                                break;
                        }
                    }
                }));
            }
        }

        private void Use()
        {
            this.Enabled = !this.Enabled;
        }

        private void FixedUpdate()
        {
            if (this.Enabled && this.Mode == Modes.Controllable)
            {
                this.body.AddForce(((Vector2)(Camera.main.ScreenToWorldPoint(Input.mousePosition) - this.transform.position).normalized + UnityEngine.Random.insideUnitCircle * 0.12f) * 12f);
                this.body.drag =  Mathf.Clamp(10f / Vector2.Distance(Camera.main.ScreenToWorldPoint(Input.mousePosition), this.transform.position), 0f, 10f);
                this.body.angularVelocity = Mathf.Lerp(this.body.angularVelocity, 1575f, Time.deltaTime * 15f);
            }
            
            if (this.Mode == Modes.Boomerang)
            {
                if (this.body.velocity.magnitude <= 3f)
                {
                    return;
                }
                this.body.velocity = Vector2.Lerp(this.body.velocity, new Vector2(Mathf.Clamp(-this.body.angularVelocity, -1, 1) * 7f, this.body.velocity.y), (Time.deltaTime * (Mathf.Abs(this.body.angularVelocity) / 500f)) * 2f); //Try AddForce as well!
            }
        }

        private void Lodged(Stabbing Stab)
        {
            this.Enabled = false;
            this.body.drag = 0f;
            if (this.body.velocity.magnitude <= 15f)
            {
                this.body.velocity *= 4f;
            }
            Debug.Log("Stab");
            switch (this.Mode)
            {
                case Modes.Electric:
                    this.StartCoroutine(ElectricShock(Stab.victim));
                    break;

                case Modes.Explosive:
                    this.StartCoroutine(ExplosiveBoom());
                    break;

                case Modes.Pizza:
                    this.SpawnPizza(Stab.victim.transform.position);
                    break;

                default:
                    break;
            }
        }

        private void SpawnPizza(Vector3 pos)
        {
            GameObject Pizza = ModAPI.CreatePhysicalObject("Pizza", ResourceStorage.Sprite_Pizza);
            PhysicalBehaviour phys = Pizza.GetComponent<PhysicalBehaviour>();
            phys.Properties = ModAPI.FindPhysicalProperties("Soft");
            phys.InitialMass /= 3f;
            phys.rigidbody.mass /= 3f;
            Pizza.transform.position = new Vector2(pos.x, pos.y + 25f);
            GameObject.Destroy(Pizza.GetComponent<Collider2D>());
            Pizza.AddComponent<CircleCollider2D>();
        }

        private IEnumerator RaiseProngs()
        {
            while ((Vector2)this.Prongs.localPosition != new Vector2(0f, ModAPI.PixelSize))
            {
                this.Prongs.localPosition = Vector2.Lerp(this.Prongs.localPosition, new Vector2(0f, ModAPI.PixelSize), Time.deltaTime * 6f);
                yield return new WaitForEndOfFrame();
            }
            Debug.Log("Prongs Up");
            yield return new WaitForEndOfFrame();
        }

        private IEnumerator LowerProngs()
        {
            while ((Vector2)this.Prongs.localPosition != Vector2.zero)
            {
                this.Prongs.localPosition = Vector2.Lerp(this.Prongs.localPosition, Vector2.zero, Time.deltaTime * 6f);
                yield return new WaitForEndOfFrame();
            }
            Debug.Log("Prongs Down");
            yield return new WaitForEndOfFrame();
        }

        private IEnumerator ElectricShock(PhysicalBehaviour other)
        {
            LimbBehaviour Limb;
            if (other.gameObject && other.gameObject.TryGetComponent<LimbBehaviour>(out Limb)) //Sadness
            {
                if (!this.Aud.isPlaying)
                {
                    this.Aud.clip = ResourceStorage.Sound_ElectricBatarangSFX;
                    this.Aud.Play();
                }
                this.PhysicalBehaviour.ConductOverride = false;
                this.PhysicalBehaviour.Charge = 5f;
                this.ElectricStuff.GetComponent<ParticleSystem>().Play();
                yield return new WaitForSeconds(1f);
                this.PhysicalBehaviour.Charge = 25f;
                Limb.Person.ShockLevel += 0.7f;
                Limb.Person.Consciousness = 0f;
                foreach (var lime in Limb.Person.Limbs)
                {
                    lime.Numbness += 0.7f;
                }
                yield return new WaitForSeconds(1f);
                this.ElectricStuff.GetComponent<ParticleSystem>().Stop();
                this.PhysicalBehaviour.ConductOverride = true;
            }
            yield return new WaitForEndOfFrame();
        }

        private IEnumerator ExplosiveBoom()
        {
            if (!this.Aud.isPlaying)
            {
                this.Aud.clip = ResourceStorage.Sound_ExplosiveBatarangSFX;
                this.Aud.Play();
            }
            this.LightSprite.material.SetFloat("_GlowIntensity", 0.9f);
            LightSprite OtherLight = ModAPI.CreateLight(this.LightSprite.transform, new Color(1f, 0.4f, 0f, 1f), 0.5f, 2f);
            yield return new WaitForSeconds(2.75f);
            ExplosionCreator.CreateFragmentationExplosion(40U, this.transform.position, 1.2f, 2f, true, false);
            this.PhysicalBehaviour.Disintegrate();
            yield return new WaitForEndOfFrame();
        }
    }

    public class Boomeranger : MonoBehaviour
    {
        private void FixedUpdate()
        {
            if (gameObject.GetComponent<Rigidbody2D>().velocity.magnitude <= 3f)
            {
                return;
            }
            gameObject.GetComponent<Rigidbody2D>().velocity = Vector2.Lerp(gameObject.GetComponent<Rigidbody2D>().velocity, new Vector2(Mathf.Clamp(-gameObject.GetComponent<Rigidbody2D>().angularVelocity, -1, 1) * 7f, gameObject.GetComponent<Rigidbody2D>().velocity.y), (Time.deltaTime * (Mathf.Abs(gameObject.GetComponent<Rigidbody2D>().angularVelocity) / 500f)) * 2f); //Try AddForce as well!
        }
    }
}
