﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;
namespace DCPP
{
    public class ButterBlade : MonoBehaviour
    {
        PolygonCollider2D Collider;
        PolygonCollider2D Trigger;
        public AudioSource audioSource;
        public AudioClip[] audioClips;
        Dictionary<LimbBehaviour,FrictionJoint2D> CurrentlyBuddered = new Dictionary<LimbBehaviour, FrictionJoint2D>();
        public void Start()
        {
            Collider = gameObject.GetComponent<PolygonCollider2D>();

            ModAPI.OnItemSpawned += ModAPI_OnItemSpawned;

            foreach (var item in FindObjectsOfType<LimbBehaviour>())
            {
                Physics2D.IgnoreCollision(Collider, item.GetComponent<Collider2D>());
            }

            Trigger = gameObject.AddComponent<PolygonCollider2D>();
            Trigger.isTrigger = true;
            Trigger.points = Collider.points;

            audioSource = gameObject.AddComponent<AudioSource>();
            audioSource.spatialBlend = 1;
            Global.main.AddAudioSource(audioSource);
        }

        private void ModAPI_OnItemSpawned(object sender, UserSpawnEventArgs e)
        {
            foreach (var item in e.Instance.GetComponentsInChildren<LimbBehaviour>())
            {
                Physics2D.IgnoreCollision(Collider, item.GetComponent<Collider2D>());
            }
        }

        void OnTriggerStay2D(Collider2D other)
        {
            if(other.GetComponent<LimbBehaviour>() != null && GetComponent<Rigidbody2D>().velocity.magnitude > 0.01)
            {
                other.gameObject.GetComponent<LimbBehaviour>().Shot(new Shot() { cartridge = ModAPI.FindCartridge("50 AE"), damage = 5, normal = transform.up, point = other.ClosestPoint(transform.position), triggersExplosiveOverride = false });
                other.gameObject.GetComponent<LimbBehaviour>().CirculationBehaviour.Cut(other.ClosestPoint(transform.position),other.ClosestPoint(transform.position) - new Vector2(transform.position.x,transform.position.y));
                if(!CurrentlyBuddered.ContainsKey(other.GetComponent<LimbBehaviour>()))
                {
                    var joint = other.gameObject.AddComponent<FrictionJoint2D>();
                    joint.connectedBody = GetComponent<Rigidbody2D>();
                    joint.anchor = Vector2.zero;
                    joint.maxForce = 440f;
                    joint.maxTorque = 440f;
                    CurrentlyBuddered.Add(other.GetComponent<LimbBehaviour>(), joint);
                }
                audioSource.clip = audioClips[UnityEngine.Random.Range(0, audioClips.Length)];
                audioSource.pitch = UnityEngine.Random.Range(0.9f, 1.1f);
                audioSource.Play();
            }
        }

        void OnTriggerExit2D(Collider2D other)
        {
            if (other.GetComponent<LimbBehaviour>() != null)
            {
                if (CurrentlyBuddered.ContainsKey(other.GetComponent<LimbBehaviour>()))
                {
                    Destroy(CurrentlyBuddered[other.GetComponent<LimbBehaviour>()]);
                    CurrentlyBuddered.Remove(other.GetComponent<LimbBehaviour>());
                }
            }
        }

        public void OnDestroy()
        {
            ModAPI.OnItemSpawned -= ModAPI_OnItemSpawned;
        }

    }
}
