﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;

namespace DCPP
{
    public class GrapnelGun : MonoBehaviour, Messages.IUse, Messages.IOnGripped, Messages.IOnDrop
    {
        private Transform Hook;
        private DistanceJoint2D Joint;
        private CableBehaviour Cable;
        private bool Detached;
        private bool Retracting;
        private FixedJoint2D Holder;
        private Vector2 OrigPos;
        private LimbBehaviour Head;
        private AudioSource Aud;

        private void Awake()
        {
            this.Hook = ModAPI.CreatePhysicalObject("Hook", ResourceStorage.Sprite_Hook).transform;
            this.Hook.position = (Vector2)this.transform.position + new Vector2(0.3f * this.transform.localScale.x, 0f);
            this.Hook.localScale = new Vector2(this.transform.localScale.x, 1f);
            this.Hook.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 0.5f;
            this.Hook.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.1f;
            this.Hook.gameObject.GetComponent<Rigidbody2D>().mass = 0.1f;
            this.Joint = this.gameObject.AddComponent<DistanceJoint2D>();
            this.Joint.connectedBody = this.Hook.gameObject.GetComponent<Rigidbody2D>();
            this.Joint.autoConfigureDistance = false;
            this.Joint.maxDistanceOnly = true;
            this.Joint.autoConfigureConnectedAnchor = false;
            this.Joint.enableCollision = true;
            this.Joint.distance = 0f;
            this.Joint.connectedAnchor = new Vector2(-0.025f, 0f);
            this.Joint.anchor = new Vector2(0.18f, 0.045f);
            this.Joint.enabled = false;
            this.Cable = this.gameObject.AddComponent<CableBehaviour>();
            this.Cable.typedJoint = this.Joint;
            this.Cable.WireColor = Color.clear;
            this.Cable.WireWidth = 0.0175f;
            this.Cable.WireMaterial = Resources.Load<Material>("Materials/Cable");
            this.Holder = this.gameObject.AddComponent<FixedJoint2D>();
            this.Holder.connectedBody = this.Hook.gameObject.GetComponent<Rigidbody2D>();
            this.Holder.enableCollision = false;
            this.Holder.autoConfigureConnectedAnchor = false;
            this.Holder.connectedAnchor = new Vector2(-0.025f, 0f);
            this.Holder.anchor = new Vector2(0.18f, 0.045f);
            this.OrigPos = this.transform.InverseTransformPoint(this.Hook.position);
            this.Aud = this.gameObject.AddComponent<AudioSource>();
            this.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            this.Aud.clip = ResourceStorage.Sound_GrapplePull;
            this.Aud.loop = true;
        }

        private void OnDisable()
        {
            this.Hook.gameObject.SetActive(false);
        }

        private void OnDestroy()
        {
            this.Hook.gameObject.SetActive(false);
        }

        public void Use(ActivationPropagation Prop)
        {
            if (!this.Retracting)
            {
                if (!this.Detached)
                {
                    this.Holder.enabled = false;
                    this.Joint.enabled = true;
                    this.Joint.distance = 15f;
                    this.Cable.SetColor(Color.black);
                    this.Hook.gameObject.GetComponent<Rigidbody2D>().AddForce(((this.transform.localScale.x > 0f)? this.transform.right.normalized : -this.transform.right.normalized) * 3f, ForceMode2D.Impulse);
                    this.Detached = true;
                    this.Aud.PlayOneShot(ResourceStorage.Sound_GrappleShoot);
                }
                else
                {
                    this.Retracting = true;
                    this.StartCoroutine(Retract());
                }
                return;
            }
            this.Retracting = false;
        }

        public void OnGripped(GripBehaviour grip)
        {
            if (grip.gameObject.GetComponent<LimbBehaviour>().Person.Limbs[0].NodeBehaviour.IsConnectedToRoot)
            {
                this.Head = grip.gameObject.GetComponent<LimbBehaviour>().Person.Limbs[0];
            }
        }

        public void OnDrop(GripBehaviour grip)
        {
            if (this.Head != null)
            {
                this.Head.BreakingThreshold = 5f;
            }
            this.Head = null;
        }

        private IEnumerator Retract()
        {
            if (this.Head != null)
            {
                this.Head.BreakingThreshold = 500000000f;
            }
            this.Aud.Play();
            while (Vector2.Distance(this.Hook.position, this.transform.position) >= 0.35f)
            {
                if (!this.Retracting)
                {
                    if (this.Head != null)
                    {
                        this.Head.BreakingThreshold = 5f;
                    }
                    this.Aud.Stop();
                    yield break;
                }
                this.gameObject.GetComponent<Rigidbody2D>().AddForce(((this.Hook.transform.position - this.transform.position).normalized * Time.deltaTime) * 15f * Mathf.Min(Vector2.Distance(this.Hook.transform.position, this.transform.position), 1f), ForceMode2D.Force);
                this.Joint.distance -= Time.deltaTime * 4f * Mathf.Min(Vector2.Distance(this.Hook.transform.position, this.transform.position) / 7f, 1f);
                yield return new WaitForEndOfFrame();
            }
            this.Joint.distance = 0f;
            this.Holder.enabled = true;
            this.Aud.Stop();
            this.Joint.enabled = false;
            this.Hook.rotation = this.transform.rotation;
            this.Hook.position = this.transform.TransformPoint(this.OrigPos);
            this.Retracting = false;
            this.Detached = false;
            if (this.Head != null)
            {
                this.Head.BreakingThreshold = 5;
            }
            this.Cable.SetColor(Color.clear);
            yield return new WaitForEndOfFrame();
        }
    }
}
