﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;

namespace DCPP
{
	public class EntireMechBehaviour : MonoBehaviour
    {
		public bool Dead;
		public bool OnFloor;
		public List<MechBehaviour> Limbs = new List<MechBehaviour>();
		public List<MechFootBehaviour> Feet = new List<MechFootBehaviour>();
		public List<MechBehaviour> Legs = new List<MechBehaviour>();
		public MechBodyBehaviour Body;
		private bool CanPush = true;
		public bool Walking;

		private void Start()
        {
			this.gameObject.AddComponent<HingeJointLimitAutofixBehaviour>();
        }

		private void Update()
        {
			this.OnFloor = false;
			int i = 0;
			foreach (MechBehaviour Limb in this.Limbs)
            {
				if (Limb.OnFloor)
                {
					this.OnFloor = true;
                }
            }

			foreach (MechFootBehaviour foot in this.Feet)
			{
				i++;
				if (!this.Body.Mech.Broken && foot.MechBehaviour.Floor != null && this.OnFloor && !foot.FootPoint.Intersects(foot.MechBehaviour.Floor.GetComponent<SpriteRenderer>().bounds) && i == 2)
				{
					this.StartCoroutine(RequestPush());
				}
			}
		}

		public void StartWalking()
        {
			this.Walking = true;
			this.StartCoroutine(this.WalkingCycle());
        }

		public void StopWalking()
		{
			this.Walking = false;
			this.StopCoroutine("WalkingCycle");
		}

		private IEnumerator RequestPush() //Lazy I guess
        {
			if (this.CanPush)
            {
				this.Body.gameObject.GetComponent<Rigidbody2D>().AddForce(Vector2.up / 3f, ForceMode2D.Impulse);
				this.CanPush = false;
			}
			yield return new WaitForSeconds(1f);
			this.CanPush = true;
		}

		private IEnumerator WalkingCycle()
		{
			/*this.Legs[0].ForceMultiplier += 6f;
			this.Legs[1].ForceMultiplier += 25f;
			this.Legs[2].ForceMultiplier += 6f;
			this.Legs[3].ForceMultiplier += 25f;
			this.Feet[0].MechBehaviour.ForceMultiplier += 7f;
			this.Feet[1].MechBehaviour.ForceMultiplier += 7f;
			var mot = this.Legs[0].Joint.motor; //I probably should've made a extension function for this...
			mot.maxMotorTorque += 50f;
			this.Legs[0].Joint.motor = mot;
			var mot2 = this.Legs[1].Joint.motor;
			mot2.maxMotorTorque += 120f;
			this.Legs[0].Joint.motor = mot2;
			var mot3 = this.Legs[2].Joint.motor;
			mot3.maxMotorTorque += 50f;
			this.Legs[0].Joint.motor = mot3;
			var mot4 = this.Legs[3].Joint.motor;
			mot4.maxMotorTorque += 120f;
			this.Legs[0].Joint.motor = mot4; // It would've been a lot more easier and less annoying.
			while (this.Walking)
            {
				this.Legs[0].Angle = 140f;
				this.Legs[1].Angle = 0f;
				this.Legs[2].Angle = -90f;
				this.Legs[3].Angle = 0f;
				this.Feet[1].MechBehaviour.Angle = 35f;
				this.Feet[0].MechBehaviour.Angle = 0f;
				yield return new WaitForSeconds(1f);
				this.Legs[0].Angle = 0f;
				this.Legs[1].Angle = 140f;
				this.Legs[2].Angle = 0f;
				this.Legs[3].Angle = -90f;
				this.Feet[1].MechBehaviour.Angle = 0f;
				this.Feet[0].MechBehaviour.Angle = 35f;
				yield return new WaitForSeconds(1f);
            }
			this.Legs[0].Angle = this.Legs[0].Joint.referenceAngle;
			this.Legs[1].Angle = this.Legs[1].Joint.referenceAngle;
			this.Legs[2].Angle = this.Legs[2].Joint.referenceAngle;
			this.Legs[3].Angle = this.Legs[3].Joint.referenceAngle;
			this.Feet[0].MechBehaviour.Angle = this.Feet[0].MechBehaviour.Joint.referenceAngle;
			this.Feet[1].MechBehaviour.Angle = this.Feet[1].MechBehaviour.Joint.referenceAngle;
			this.Legs[0].ForceMultiplier -= 6f;
			this.Legs[1].ForceMultiplier -= 25f;
			this.Legs[2].ForceMultiplier -= 6f;
			this.Legs[3].ForceMultiplier -= 25f;
			this.Feet[0].MechBehaviour.ForceMultiplier -= 7f;
			this.Feet[1].MechBehaviour.ForceMultiplier -= 7f;
			var mote = this.Legs[0].Joint.motor;
			mote.maxMotorTorque -= 50f;
			this.Legs[0].Joint.motor = mote;
			var mote2 = this.Legs[1].Joint.motor;
			mote2.maxMotorTorque -= 120f;
			this.Legs[0].Joint.motor = mote2;
			var mote3 = this.Legs[2].Joint.motor;
			mote3.maxMotorTorque -= 50f;
			this.Legs[0].Joint.motor = mote3;
			var mote4 = this.Legs[3].Joint.motor;
			mote4.maxMotorTorque -= 120f;
			this.Legs[0].Joint.motor = mote4;*/
			while (this.Walking)
            {
				if (this.Feet[1].MechBehaviour.Broken)
                {
					this.StopWalking();
					yield break;
                }
				this.Feet[1].gameObject.GetComponent<Rigidbody2D>().AddForce(new Vector2(0.45f, 0.22f) * 30f * this.transform.localScale.x, ForceMode2D.Impulse);
				yield return new WaitForSeconds(0.7f);
				if (this.Feet[0].MechBehaviour.Broken)
				{
					this.StopWalking();
					yield break;
				}
				this.Feet[0].gameObject.GetComponent<Rigidbody2D>().AddForce(new Vector2(0.45f, 0.22f) * 30f * this.transform.localScale.x, ForceMode2D.Impulse);
				yield return new WaitForSeconds(0.7f);
				if (this.Feet[0].MechBehaviour.Broken)
				{
					this.StopWalking();
					yield break;
				}
				this.Feet[0].gameObject.GetComponent<Rigidbody2D>().AddForce(new Vector2(0.45f, 0.22f) * 30f * this.transform.localScale.x, ForceMode2D.Impulse);
				yield return new WaitForSeconds(0.7f);
				if (this.Feet[1].MechBehaviour.Broken)
				{
					this.StopWalking();
					yield break;
				}
				this.Feet[1].gameObject.GetComponent<Rigidbody2D>().AddForce(new Vector2(0.45f, 0.22f) * 30f * this.transform.localScale.x, ForceMode2D.Impulse);
				yield return new WaitForSeconds(0.7f);
			}
			yield return new WaitForEndOfFrame();
		}
	}

	public class MechBehaviour : MonoBehaviour, Messages.IShot, Messages.IOnFragmentHit, Messages.IStabbed, Messages.ISlice
    {
		public EntireMechBehaviour Person;
		public HingeJoint2D Joint;
		public bool OnFloor;
		public GameObject Floor;
		public Vector3 FloorPos;
		public bool Override;
		public float ForceMultiplier = 1f;
		public float Angle = 0f;
		public bool Broken;
		private Material Mat;
		private Material OrigMat;
		private float Damage;
		private float Health;

		private void Start()
        {
			if (this.gameObject.GetComponent<HingeJoint2D>() != null)
			{
				this.Joint = this.gameObject.GetComponent<HingeJoint2D>();
				this.Angle = this.Joint.referenceAngle;
			}
			this.Health = 6500f + (this.gameObject.GetComponent<SpriteRenderer>().size.sqrMagnitude * 150f);
			if (this.gameObject.GetComponent<MechBodyBehaviour>() != null)
            {
				this.Health += 2000f;
            }
			this.gameObject.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Insert(0, new ContextMenuButton("Check Damage", "Check Damage", "Check how much damage the bot has taken compared to it's heath. This is shown as a notification.", new UnityAction[1]
			{
				(UnityAction) (() =>
				{
					ModAPI.Notify("<color=red><size=40>" + this.Damage + " / " + this.Health + "<color=red><size=40>");
				})
			}));
			this.OrigMat = this.gameObject.GetComponent<SpriteRenderer>().material;
		}

		private void FixedUpdate()
        {
			if (this.Floor == null)
            {
				this.OnFloor = false;
            }
			if (this.Joint && (this.Person.OnFloor || this.Person.Walking) && !this.Broken)
            {
				this.Joint.useMotor = true;
				JointMotor2D motor = this.Joint.motor;
				motor.motorSpeed = Mathf.DeltaAngle(this.Joint.jointAngle, this.Angle) * 8f * this.ForceMultiplier;
				this.Joint.motor = motor;
			}
			else if (this.Joint && !this.Override)
            {
				this.Joint.useMotor = false;
            }
		}

		public void OnCollisionEnter2D(Collision2D col)
        {
			if (!this.OnFloor && col.gameObject != null)
            {
				this.OnFloor = true;
				this.Floor = col.gameObject;
				this.FloorPos = col.GetContact(0).point;
			}
			if (col.gameObject.GetComponent<WrenchBehaviour>())
            {
				Fix();
            }
        }

		public void OnCollisionExit2D(Collision2D col)
		{
			if (this.Floor == null)
            {
				this.OnFloor = false;
			}
			if (col == null)
			{
				return;
			}
			if (col.gameObject == this.Floor)
            {
				this.OnFloor = false;
			}
		}

		public void Shot(Shot shot)
        {
			this.Damage += shot.damage / 3f;
			TestIfBreak();
		}

		public void OnFragmentHit(float dam)
		{
			this.Damage += dam * 2f;
			TestIfBreak();
		}

		public void Stabbed(Stabbing stab)
        {
			this.Damage += 250f + stab.stabber.spriteRenderer.size.magnitude * 10f;
			TestIfBreak();
		}

		public void Slice()
        {
			this.Damage += 500f;
			TestIfBreak();
		}

		public void TestIfBreak()
        {
			if (this.Damage > this.Health && !this.Broken)
            {
				this.Broken = true;
				this.gameObject.SendMessage("Broke", null, SendMessageOptions.DontRequireReceiver);
				ModAPI.CreateParticleEffect("Spark", this.transform.position);
				var sprite = this.gameObject.GetComponent<SpriteRenderer>();
				this.Mat = new Material(Shader.Find("Unlit/BrokenElectronics"));
				this.Mat.SetTexture("_DamageMap", ResourceStorage.Texture_DamageMask);
				this.Mat.SetTexture("_BrokenElectronics", ResourceStorage.Texture_ElectronicMask);
				this.Mat.SetTexture("_MainTex", sprite.sprite.texture);
				this.Mat.SetVector("_Offset", new Vector4(UnityEngine.Random.value * 1000f, UnityEngine.Random.value * 1000f));
				sprite.material = this.Mat;
				if (this.gameObject.name.Contains("Body"))
                {
					foreach (var limb in this.Person.Limbs)
                    {
						limb.Damage = Mathf.Infinity;
						limb.TestIfBreak();
					}
                }
			}
        }

		private void Fix()
        {
			this.Damage = 0f;
			var sprite = this.gameObject.GetComponent<SpriteRenderer>();
			sprite.material = this.OrigMat;
			this.Broken = false;
		}
	}

	public class MechGrabBehaviour : MonoBehaviour, Messages.IUse
    {
		FixedJoint2D joints;
		private bool Used;
		private BoxCollider2D SwordPos;
		private GameObject Sword;
		private bool SwordUsed;
		public EntireMechBehaviour Person;
		private GameObject trail;
		public MechBehaviour Mech;

		private void Start()
        {
			this.Mech = this.gameObject.GetComponent<MechBehaviour>();
			var SwordSpr = new GameObject("SwordPos", new Type[] 
			{
				typeof(SpriteRenderer)
			}).GetComponent<SpriteRenderer>();
			SwordSpr.sprite = ResourceStorage.Sprite_SwordCol;
			this.SwordPos = SwordSpr.gameObject.AddComponent<BoxCollider2D>();
			this.SwordPos.isTrigger = true;
			SwordSpr.transform.SetParent(this.transform);
			SwordSpr.color = Color.clear;
			this.SwordPos.transform.localPosition = new Vector3(-0.32f, 0f);
			this.Sword = ModAPI.CreatePhysicalObject("Sword", ResourceStorage.Sprite_Sword);
			this.Sword.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindSpawnable("Sword").Prefab.GetComponent<PhysicalBehaviour>().Properties;
			this.Sword.transform.SetParent(this.transform);
			this.Sword.transform.localPosition = new Vector3(-0.318f, -0.6f, 2f);
			this.Sword.transform.localScale = new Vector2(this.transform.root.localScale.x, 1f);
			this.Sword.transform.rotation = Quaternion.Euler(this.transform.eulerAngles.x, this.transform.eulerAngles.y, this.transform.eulerAngles.z + 180);
			this.Sword.AddComponent<FixedJoint2D>().connectedBody = this.gameObject.GetComponent<Rigidbody2D>();
			this.Sword.GetComponent<FixedJoint2D>().frequency = 250f;
			this.Sword.GetComponent<FixedJoint2D>().dampingRatio = -15f;
			this.trail = UnityEngine.Object.Instantiate<GameObject>(ModAPI.FindSpawnable("Energy Sword").Prefab.GetComponent<EnergySwordBehaviour>().MotionBlur, this.Sword.transform);
			trail.transform.position = this.Sword.transform.position;
			trail.transform.rotation = this.Sword.transform.rotation;
			var boh = trail.GetComponent<EnergySwordMotionBlur>();
			boh.Life = 0.25f;
			var alpha = boh.ColorOverMesh.alphaKeys;
			var col = new GradientColorKey[]
			{
				new GradientColorKey(),
				new GradientColorKey()
			};
			col[0].color = new Color(0f, 1f, 0f, 1f);
			col[0].time = 0f;
			col[1].color = new Color(0f, 1f, 0f, 0f);
			col[1].time = 1f;
			boh.ColorOverMesh = new Gradient()
            {
				alphaKeys = alpha,
				colorKeys = col
            };
			boh.Rigidbody = this.Sword.GetComponent<Rigidbody2D>();
			boh.A = new GameObject().transform;
			boh.B = new GameObject().transform;
			boh.A.position = this.Sword.transform.TransformPoint(-0.12f, 0.76f, 0f);
			boh.B.position = this.Sword.transform.TransformPoint(-0.12f, -0.65f, 0f);
			boh.A.SetParent(this.Sword.transform);
			boh.B.SetParent(this.Sword.transform);
			this.Sword.AddComponent<KryptoniteBehaviour>();
			this.Sword.AddComponent<KrypSwordBehaviour>();
			trail.SetActive(false);
			this.Sword.SetActive(false);
			this.Sword.GetComponent<SpriteRenderer>().material = ModAPI.FindMaterial("VeryBright");
			this.Sword.GetComponent<SpriteRenderer>().color = new Color(0.1f, 0.1f, 0.1f, 0.7f);
		}

		public void Use(ActivationPropagation Prop)
        {
			if (this.Mech.Broken)
            {
				return;
            }
			if (this.SwordPos.OverlapPoint(Global.main.MousePosition))
            {
				this.SwordUsed = !this.SwordUsed;
				if (this.SwordUsed)
                {
					foreach (var limb in this.Person.Limbs)
					{
						Physics2D.IgnoreCollision(this.Sword.GetComponent<Collider2D>(), limb.gameObject.GetComponent<Collider2D>());
					}
					this.Sword.transform.localPosition = new Vector3(-0.318f, -0.6f, 2f);
					this.Sword.transform.rotation = Quaternion.Euler(this.transform.eulerAngles.x, this.transform.eulerAngles.y, this.transform.eulerAngles.z + 180);
					trail.transform.position = this.Sword.transform.position; //Safety Precaution
					this.Sword.SetActive(true);
					this.trail.SetActive(true);
					this.Sword.GetComponent<Rigidbody2D>().velocity = Vector2.zero;
					this.Sword.GetComponent<Rigidbody2D>().angularVelocity = 0f;
					AudioSource.PlayClipAtPoint(ResourceStorage.Sound_Draw, this.Sword.transform.position);
					return;
                }
				this.Sword.SetActive(false);
				this.trail.SetActive(false);
				foreach (var pen in this.Sword.GetComponent<PhysicalBehaviour>().penetrations)
                {
					if(!pen.Joint)
					{
						return;
					}
					GameObject.Destroy(pen.Joint);
					pen.Active = false;
					Collider2D b = pen.Stabber.colliders[0];
					foreach (Collider2D collider2D in pen.RelevantColliders)
					{
						if (collider2D)
						{
							Physics2D.IgnoreCollision(collider2D, b, false);
						}
					}
				}
				return;
			}
			if (this.Used)
            {
				foreach (var col in this.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
				{
					foreach (var col2 in this.joints.connectedBody.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
					{
						Physics2D.IgnoreCollision(col, col2, false);
					}
				}
				GameObject.Destroy(this.joints);
				this.Used = false;
				return;
            }
			var cast = Physics2D.OverlapCircleAll(this.transform.TransformPoint(new Vector2(0f, -0.8f)), 0.8f, LayerMask.GetMask("Objects"), -int.MaxValue, int.MaxValue);
			Debug.Log(cast.Length.ToString());
			foreach (var obj in cast)
            {
				if (obj.transform.root == this.transform.root)
                {
					continue;
                }
				this.Used = true;
				this.joints = this.gameObject.AddComponent<FixedJoint2D>();
				this.joints.connectedBody = obj.attachedRigidbody;
				this.joints.anchor = new Vector2(0f, -0.9f);
				foreach (var col in this.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
                {
					foreach (var col2 in obj.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
                    {
						Physics2D.IgnoreCollision(col, col2);
					}
                }
				return;
			}
        }

		private void OnDisable()
        {
			this.Sword.SetActive(false);
			trail.SetActive(false);
		}

		public void Broke()
        {
			if (this.joints != null)
            {
				foreach (var col in this.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
				{
					foreach (var col2 in this.joints.connectedBody.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
					{
						Physics2D.IgnoreCollision(col, col2, false);
					}
				}
				GameObject.Destroy(this.joints);
			}
			this.Used = false;
		}

		private void OnDestroy()
        {
			GameObject.Destroy(this.Sword);
        }
    }

	public class KrypSwordBehaviour : MonoBehaviour, Messages.ILodged, Messages.IDislodged
    {
		public void Lodged(Stabbing stab)
        {
			if (stab.victim.Deletable && stab.victim.gameObject.tag == "Limb")
            {
				foreach (var obj in stab.victim.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
				{
					foreach (var obj2 in this.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
					{
						Physics2D.IgnoreCollision(obj, obj2);
					}
				}
			}
        }

		public void Dislodged(PhysicalBehaviour.Penetration pen)
		{
			if (pen.Victim.Deletable && pen.Victim.gameObject.tag == "Limb")
			{
				foreach (var obj in pen.Victim.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
				{
					foreach (var obj2 in this.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
					{
						Physics2D.IgnoreCollision(obj, obj2, false);
					}
				}
			}
		}
	}

	public class MechLaserBehaviour : MonoBehaviour, Messages.IUse
	{
		private BoxCollider2D RocketPos;
		private bool Used = false;
		public EntireMechBehaviour Person;
		private LineRenderer line;
		public MechBehaviour Mech;
		private AudioSource Source;

		private void Start()
		{
			this.Mech = this.gameObject.GetComponent<MechBehaviour>();
			var SwordSpr = new GameObject("SwordPos", new Type[]
			{
				typeof(SpriteRenderer)
			}).GetComponent<SpriteRenderer>();
			SwordSpr.sprite = ResourceStorage.Sprite_RocketCol;
			this.RocketPos = SwordSpr.gameObject.AddComponent<BoxCollider2D>();
			this.RocketPos.isTrigger = true;
			SwordSpr.transform.SetParent(this.transform);
			SwordSpr.color = Color.clear;
			this.RocketPos.transform.localPosition = new Vector3(0.32f, 0f);
			this.line = this.gameObject.AddComponent<LineRenderer>();
			this.line.startColor = new Color(0f, 0.25f, 0f);
			this.line.endColor = new Color(0f, 0.25f, 0f);
			this.line.startWidth = ModAPI.PixelSize * 10f;
			this.line.endWidth = ModAPI.PixelSize * 10f;
			this.line.material = UnityEngine.Object.Instantiate<Material>(ModAPI.FindMaterial("VeryBright"));
			this.line.material.mainTexture = ResourceStorage.Texture_Laser;
			this.line.textureMode = LineTextureMode.Tile;
			this.line.sortingOrder = this.gameObject.GetComponent<SpriteRenderer>().sortingOrder + 1;
			this.line.sortingLayerID = this.gameObject.GetComponent<SpriteRenderer>().sortingLayerID;
			this.line.positionCount = 2;
			this.line.enabled = false;
			this.Source = SwordSpr.gameObject.AddComponent<AudioSource>();
			this.Source.loop = true;
			this.Source.clip = ResourceStorage.Sound_SLaserConstant;
			SwordSpr.gameObject.AddComponent<AudioDistortionFilter>().distortionLevel = 0.55f;
			SwordSpr.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
		}

		public void Use(ActivationPropagation Prop)
		{
			if (this.Mech.Broken)
            {
				return;
            }
			if (this.RocketPos.OverlapPoint(Global.main.MousePosition))
			{
				GameObject Rocket = UnityEngine.Object.Instantiate<GameObject>(ModAPI.FindSpawnable("Rocket Launcher").Prefab.GetComponent<RocketLauncherBehaviour>().Projectile, this.transform.TransformPoint(new Vector2(0.32f, -0.78f)), this.transform.rotation);
				Rocket.transform.up = this.transform.right;
				Rocket.GetComponentInChildren<SpriteRenderer>().sprite = ResourceStorage.Sprite_Rocket;
				this.gameObject.GetComponent<PhysicalBehaviour>().PlayClipOnce(ModAPI.FindSpawnable("Rocket Launcher").Prefab.GetComponent<RocketLauncherBehaviour>().AudioSource.clip);
				return;
			}
			this.Used = !this.Used;
			if (this.Used)
			{
				this.line.enabled = true;
				this.Source.PlayOneShot(ResourceStorage.Sound_LaserOn, 2f); //Placeholder Sound
				this.Source.Play();
				return;
			}
			this.line.enabled = false;
			this.Source.Stop();
			this.Source.PlayOneShot(ResourceStorage.Sound_SLaserOff, 2f); //Placeholder Sound
		}

		private void Update()
		{
			PhysicalBehaviour Phys;
			LimbBehaviour Limb;
			if (this.Used && !this.Mech.Broken)
			{
				this.line.material.mainTextureOffset -= new Vector2((Time.deltaTime * 15f), 0f);
				line.SetPosition(0, this.transform.TransformPoint(new Vector2(-0.12f, -0.75f)));
				RaycastHit2D hit = Physics2D.Raycast(this.transform.TransformPoint(new Vector2(-0.12f, -0.75f)), -this.transform.up, 25000f, LayerMask.GetMask(new string[] { "Objects", "Bounds", "CollidingDebris" }));
				if (hit.transform && hit.transform.root.gameObject != this.transform.root.gameObject && hit.transform.gameObject.TryGetComponent<PhysicalBehaviour>(out Phys))
				{
					Phys.Temperature = Mathf.Lerp(Phys.Temperature, 5000f, Time.deltaTime / 2f);
					Phys.rigidbody.AddForceAtPosition((hit.normal * Time.timeScale) * -0.15f, hit.point, ForceMode2D.Impulse);
					if (Phys.gameObject.TryGetComponent<LimbBehaviour>(out Limb))
					{
						Limb.Damage(8f * Time.timeScale);
						Limb.Person.AddPain(8f * Time.timeScale);
						Limb.SkinMaterialHandler.ImpactDamage(8f * Time.timeScale, hit.point);
					}
					Phys.Charge += Time.deltaTime * 2f;
				}
				line.SetPosition(1, hit.point);
			}
			
		}

		public void Broke()
        {
			this.Used = false;
			this.line.enabled = false;
			this.Source.Stop();
			this.Source.PlayOneShot(ResourceStorage.Sound_SLaserOff, 2f); //Placeholder Sound
		}
	}

	public class MechFootBehaviour : MonoBehaviour
    {
		public MechBehaviour MechBehaviour;
		public EntireMechBehaviour Person;
		public Bounds FootPoint;

		private void Start()
        {
			this.MechBehaviour = this.gameObject.GetComponent<MechBehaviour>();
			this.Person.Feet.Add(this);
			this.FootPoint = new Bounds()
			{
				size = new Vector2(0.3f, 0.3f),
				center = this.transform.TransformPoint(new Vector2(0f, -0.3f))
			};
			//this.MechBehaviour.Override = true;
        }

		private void Update()
        {
			this.FootPoint.center = this.transform.TransformPoint(new Vector2(0f, -0.3f));
		}
    }

    public class MechBodyBehaviour : MonoBehaviour
    {
		private float FakeUprightForce = 3.65f;
		public EntireMechBehaviour Person;
		public MechBehaviour Mech;
		public GameObject FrontFoot;
		public GameObject BackFoot;
		private Vector2 PrevDotFront;
		private Vector2 PrevDotBack;

		private void Start()
        {
			this.Person.Body = this;
			this.Mech = this.gameObject.GetComponent<MechBehaviour>();
			this.gameObject.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("Walk", "Toggle Walking", "Makes the bot walk.", new UnityAction[1]
			{
				(UnityAction) (() =>
				{
					if (!this.Person.Walking)
                    {
						this.Person.StartWalking();
					}
                    else
                    {
						this.Person.StopWalking();
					}
				})
			}));
		}

		private void FixedUpdate()
        {
			if (!this.Person.OnFloor || this.Mech.Broken)
            {
				return;
            }
			PhysicalBehaviour PhysicalBehaviour = this.gameObject.GetComponent<PhysicalBehaviour>();
			float num = Mathf.DeltaAngle(base.transform.eulerAngles.z, 0f) * this.FakeUprightForce;
			if (PhysicalBehaviour.rigidbody.bodyType != RigidbodyType2D.Dynamic)
			{
				return;
			}
			if (Vector2.Dot(base.transform.up, Vector2.down) > 0.1f)
			{
				return;
			}
			if (float.IsNaN(num))
			{
				return;
			}
			PhysicalBehaviour.rigidbody.AddTorque(num * 1.2f);
		}
    }
}
