﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Rendering;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;
namespace DCPP
{
    public class HatBehaviour : MonoBehaviour
    {
        public Sprite Hat;
        public Vector2 HatOffset = Vector2.zero;
        public float HatFriction = 1f;
        private GameObject HatObject;
        private FrictionJoint2D joint;

        private void Start()
        {
            HatObject = ModAPI.CreatePhysicalObject("Hat", Hat);
            HatObject.transform.parent = transform;
            HatObject.transform.localPosition = HatOffset;
            HatObject.transform.localScale = Vector3.one;
            HatObject.GetComponent<PhysicalBehaviour>().InitialMass = 0.01f;
            HatObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.01f;
            HatObject.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Insulator");
            HatObject.GetComponent<Rigidbody2D>().mass = 0.01f;
            HatObject.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
            joint = HatObject.AddComponent<FrictionJoint2D>();
            joint.connectedBody = gameObject.GetComponent<Rigidbody2D>();
            joint.anchor = Vector2.zero;
            joint.maxForce = HatFriction;
            joint.maxTorque = HatFriction;
            joint.breakForce = HatFriction * 5f;
            joint.breakTorque = HatFriction * 5f;
            HatObject.transform.parent = transform.root;
        }

        private void FixedUpdate() //Runs at a lower framerate (50fps if I recall) so therefore check isn't done as much
        {
            if (!gameObject.GetComponent<Collider2D>().OverlapPoint(HatObject.transform.TransformPoint(0f, Hat.bounds.min.y, 0f)))
            {
                GameObject.Destroy(joint);
            }
        }
    }

    public class CondimentBehaviour : MonoBehaviour
    {
        public Material mat;
        public AudioClip splat;
        private AudioSource aud;
        private float time;
        private int swapTime;
        private bool must;
        private ParticleSystem part;

        private void Start()
        {
            //Instantiating Particlesystem should instantiate the gameobject
            part = Instantiate<ParticleSystem>(ModAPI.FindSpawnable("Blood Tank").Prefab.GetComponent<BloodTankBehaviour>().BloodDrainParticles, transform, false);
            part.transform.localPosition = new Vector2(ModAPI.PixelSize * 14f, ModAPI.PixelSize);
            ParticleSystem.EmissionModule emission2 = part.emission;
            ParticleSystem.MainModule main = part.main;
            main.startColor = Color.red;
            main.startSpeedMultiplier = 25f;
            emission2.rateOverTimeMultiplier = 50f;
            aud = gameObject.AddComponent<AudioSource>();
            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            aud.spatialBlend = 1f;
            part.Stop();
        }

        private void FixedUpdate()
        {
            if (gameObject.GetComponent<PhysicalBehaviour>().IsBeingUsedContinuously())
            {
                time += Time.fixedDeltaTime * 3f;
                swapTime++;
                if (swapTime == 5)
                {
                    aud.PlayOneShot(splat, 3f);
                    swapTime = 0;
                    var main = part.main;
                    must = !must;
                    if (!must)
                    {
                        main.startColor = Color.red;
                    }
                    else
                    {
                        main.startColor = Color.yellow;
                    }
                }
                mat.SetFloat("_Rotation", time);
                part.Play();
            }
            else
            {
                part.Stop();
            }
        }
    }
}
