﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;
namespace DCPP
{
    public class Throwable : MonoBehaviour, Messages.IDislodged
	{
        private PhysicalBehaviour PhysicalBehaviour;
        private Rigidbody2D body;
        public float ImpactRequired = 0.4f;
        private ContactPoint2D[] buffer = new ContactPoint2D[8];
		public PhysicalBehaviour.Penetration penetration;
		public float LowerLimit = -0.08f;
		public float UpperLimit = 0.13f;
		private SharpAxis Axis;


        private void Awake()
        {
            this.PhysicalBehaviour = this.gameObject.GetComponent<PhysicalBehaviour>();
            this.body = this.gameObject.GetComponent<Rigidbody2D>();
        }

        private void OnCollisionEnter2D(Collision2D col)
        {
            PhysicalBehaviour Phys;
            int contacts = col.GetContacts(this.buffer);
            if (Utils.GetMinImpulse(this.buffer, contacts) < this.ImpactRequired)
            {
                return;
            }
			if (col.collider.gameObject.TryGetComponent<PhysicalBehaviour>(out Phys) && this.PhysicalBehaviour.penetrations.Count <= 0)
            {
                CreateStabWound(col, Utils.GetFirstValidContact(this.buffer, contacts));
            }
        }

		private void CreateStabWound(Collision2D collision, ContactPoint2D contact)
		{
			PhysicalBehaviour physicalBehaviour;
			if (!Global.main.PhysicalObjectsInWorldByTransform.TryGetValue(collision.transform, out physicalBehaviour))
			{
				return;
			}
			if (physicalBehaviour.Properties.Softness <= 0.01f)
			{
				return;
			}
			Vector2 normalized = collision.relativeVelocity.normalized;
			this.penetration = new PhysicalBehaviour.Penetration();
			penetration.Stabber = this.PhysicalBehaviour;
			SharpAxis sharpAxis = new SharpAxis(normalized, LowerLimit, UpperLimit, true, false);
			if (!penetration.Active)
			{
				penetration.Active = true;
				penetration.Duration = 0f;
				this.PhysicalBehaviour.rigidbody.velocity = this.PhysicalBehaviour.GetPreviousVel();
				if (physicalBehaviour.rigidbody.bodyType == RigidbodyType2D.Dynamic)
				{
					physicalBehaviour.rigidbody.velocity = physicalBehaviour.GetPreviousVel();
				}
				Vector2 vector2 = contact.point;
				vector2 = collision.collider.ClosestPoint(vector2);
				Stabbing stabbing = new Stabbing(this.PhysicalBehaviour, physicalBehaviour, contact.normal, vector2);
				physicalBehaviour.SendMessage("Stabbed", stabbing, SendMessageOptions.DontRequireReceiver);
				base.gameObject.SendMessage("Lodged", stabbing, SendMessageOptions.DontRequireReceiver);
				penetration.CurrentConnectedAnchor = physicalBehaviour.transform.InverseTransformPoint(vector2);
				this.DoStab(physicalBehaviour, sharpAxis, penetration, contact.point);
			}
		}

		private void DoStab(PhysicalBehaviour other, SharpAxis incomingSharpAxis, PhysicalBehaviour.Penetration penetration, Vector2 HitPoint)
        {
			penetration.Active = true;
			penetration.Victim = other;
			penetration.Victim.stabWoundCount += 1U;
			penetration.Victim.beingStabbedBy.Add(this.PhysicalBehaviour);
			penetration.Victim.victimPenetrations.Add(penetration);
			penetration.Axis = incomingSharpAxis;
			SliderJoint2D sliderJoint2D = base.gameObject.AddComponent<SliderJoint2D>();
			sliderJoint2D.autoConfigureConnectedAnchor = false;
			sliderJoint2D.autoConfigureAngle = false;
			sliderJoint2D.anchor = Vector2.zero;
			sliderJoint2D.breakForce = 9999999f;
			sliderJoint2D.breakTorque = 9999999f;
			sliderJoint2D.connectedAnchor = penetration.CurrentConnectedAnchor;
			sliderJoint2D.connectedBody = other.rigidbody;
			sliderJoint2D.enableCollision = false;
			sliderJoint2D.useMotor = true;
			JointMotor2D motor = sliderJoint2D.motor;
			motor.maxMotorTorque = this.PhysicalBehaviour.Properties.SharpForceThresholdMultiplier * ((1f - other.Properties.Softness) * 5f + 5f) * this.PhysicalBehaviour.Properties.LodgeStrengthMultiplier;
			sliderJoint2D.motor = motor;
			sliderJoint2D.angle = Vector2.Angle(this.gameObject.transform.position, HitPoint);
			sliderJoint2D.useLimits = true;
			sliderJoint2D.limits = new JointTranslationLimits2D
			{
				min = incomingSharpAxis.LowerLimit,
				max = incomingSharpAxis.UpperLimit
			};
			penetration.Joint = sliderJoint2D;
			Collider2D collider2D = this.gameObject.GetComponent<Collider2D>();
			foreach (Collider2D collider2D2 in other.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
			{
				if (!(collider2D2 == null) && !(collider2D == null))
				{
					IgnoreCollisionStackController.IgnoreCollisionSubstituteMethod(collider2D2, collider2D, true);
					penetration.RelevantColliders.Add(collider2D2);
				}
			}
			this.PhysicalBehaviour.penetrations.Add(penetration);
		}

		public void Dislodged(PhysicalBehaviour.Penetration penetration)
        {
			Collider2D b = this.gameObject.GetComponent<Collider2D>();
			foreach (Collider2D collider2D in penetration.Victim.transform.root.gameObject.GetComponentsInChildren<Collider2D>())
			{
				if (collider2D)
				{
					IgnoreCollisionStackController.IgnoreCollisionSubstituteMethod(collider2D, b, false);
				}
			}
		}
	}
}
