﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;
namespace DCPP
{
    //An empty class for you to copy paste and fill with your own powers!
    public class ElectroShock : PowerBase, Messages.IUseContinuous, Messages.IUse
    {
        private int time = 0;
        private AudioSource aud;
        private int Playing = 0;

        public void Start()
        {
            //We set our base info and values here
            PowerName = "Lightning";
            //We set our power activation type (Head,Chest,Hand)
            Activation = "Hand";

            this.aud = this.gameObject.AddComponent<AudioSource>();
            this.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            //this.aud.clip = MPP.elec;
            this.aud.loop = true;

            //Always keep this
            base.Start();
        }

        public void Use(ActivationPropagation prop)
        {
            if (Enabled)
            {
                this.aud.Play();
            }
        }

        public void UseContinuous(ActivationPropagation prop)
        {
            if (Enabled)
            {
                this.time += 1;
                this.Playing = 2;
                if ((this.time % 15) == 0)
                {
                    this.gameObject.layer = LayerMask.NameToLayer("CollidingDebris");
                    SimpleLightningEmitterBehaviour.Instance.Emit(this.transform.TransformPoint(0f, -0.15f, 0f), -this.transform.up, 20f, 0.15f, OnHit, new Color(1f, 1f, 1f, 0.8f), LayerMask.GetMask(new string[] { "Objects", "Bounds" }), 0, 256, 0.07f, 0.002f);
                    this.gameObject.layer = LayerMask.NameToLayer("Objects");
                }
            }
        }

        private void Update()
        {
            this.Playing -= 1;
            if (this.Playing <= 0)
            {
                this.aud.Stop();
            }
        }

        public void OnHit(PhysicalBehaviour phys, Vector2 pos)
        {
            if (Enabled)
            {
                LimbBehaviour limb;
                if (phys.gameObject.TryGetComponent<LimbBehaviour>(out limb) && this.gameObject.GetComponent<LimbBehaviour>().Person.Limbs.Contains<LimbBehaviour>(limb))
                {
                    return;
                }
                phys.charge += 15f;
                phys.Temperature += 30f;
                ModAPI.CreateParticleEffect("Spark", pos);
                ModAPI.CreateParticleEffect("BigZap", pos);
            }
        }


        /// <summary>
        /// Set up the power on a character.
        /// </summary>
        /// <param name="body">The selected limb of the user.</param>
        /// <param name="person">The PersonBehaviour of the user. (body.Person)</param>
        /// <param name="Instance">The overall instance of the user. ( body.Person.gameObject)</param>
        public static void SetUpPower(LimbBehaviour body, PersonBehaviour person, GameObject Instance)
        {
            //This checks if the person already has the power, if they don't we give it to them (duplicates are bad) NOTE:make sure to change the class to your new power class
            if (!body.GetComponent<ElectroShock>())
            {
                //Adds your new power to the person NOTE:make sure to change the class to your new power class
                body.gameObject.AddComponent<ElectroShock>();

                //This sets the power's icon, make sure you add the your new icon to the ResourceStorage NOTE:make sure to change the class to your new power class
                    body.GetComponent<ElectroShock>().PowerIcon = ResourceStorage.Sprite_ShockPowerIcon;

                foreach (var limb in person.Limbs)
                {
                    limb.PhysicalBehaviour.ForceNoCharge = true;
                }

            }
        }
    }
}
