﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;

namespace DCPP
{
    public class HeadExploderPower : PowerBase
    {
        /// Team Wilson's Power Pack Framework: Power Template
        public List<AudioClip> Screams = new List<AudioClip>();
        public AudioSource audioSource;
        public void Awake()
        {
            //We set our base info and values here
            PowerName = "Leathal Scream";
            Activation = "Head";

            audioSource = new GameObject().gameObject.AddComponent<AudioSource>();
            audioSource.transform.SetParent(transform, false);
            GameObject.FindObjectOfType<Global>().AddAudioSource(audioSource,false);
            audioSource.gameObject.AddComponent<AudioDistortionFilter>().distortionLevel = 0.75f;
            audioSource.outputAudioMixerGroup = GameObject.FindObjectOfType<Global>().SoundEffects;
            audioSource.spatialBlend = 1;
            audioSource.volume = 1;

            Screams.Add(ResourceStorage.Sound_BlackCanaryScream1);
            Screams.Add(ResourceStorage.Sound_BlackCanaryScream2);
            Screams.Add(ResourceStorage.Sound_BlackCanaryScream3);
        }
        public override void UsePower()
        {

            if (Enabled && GetComponent<LimbBehaviour>().Person.IsAlive())
            {
                //Gets all objects in a certain radius.
                Collider2D[] InTrigger = Physics2D.OverlapCircleAll(transform.position, 5);

                audioSource.PlayOneShot(Screams[UnityEngine.Random.RandomRange(0, Screams.Count)]);
                foreach (var item in InTrigger)
                {
                    //Checks if the object is a limb, is a head and doesn't belong to the user.
                    if (item.GetComponent<LimbBehaviour>() && item.GetComponent<LimbBehaviour>().HasBrain && !item.GetComponent<KyrptonGuy>() && !item.GetComponent<LimbBehaviour>().ImmuneToDamage && !GetComponent<LimbBehaviour>().Person.Limbs.Contains(item.GetComponent<LimbBehaviour>()))
                    {
                        //Creates the pop effect.
                        ModAPI.CreateParticleEffect("FuseBlown", item.transform.position);
                        //Destroys the head.
                        item.GetComponent<LimbBehaviour>().Crush();
                    }
                }
            }
        }

        public override void RemovePower()
        {
            Destroy(audioSource.gameObject);
            base.RemovePower();
        }

        /// <summary>
        /// Set up the power on a character.
        /// </summary>
        /// <param name="body">The selected limb of the user.</param>
        /// <param name="person">The PersonBehaviour of the user. (body.Person)</param>
        /// <param name="Instance">The overall instance of the user. ( body.Person.gameObject)</param>
        public static void SetUpPower(LimbBehaviour body, PersonBehaviour person, GameObject Instance)
        {
            if (!body.GetComponent<HeadExploderPower>())
            {
                body.gameObject.AddComponent<HeadExploderPower>();

                body.GetComponent<HeadExploderPower>().PowerIcon = ResourceStorage.Sprite_HeadExploderPowerIcon;

                body.gameObject.AddComponent<UseEventTrigger>().Action = () =>
                {
                    body.gameObject.GetComponent<HeadExploderPower>().UsePower();
                };
            }
        }
    }
}
