﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;
namespace DCPP
{
    public class LaserBlasterPower : PowerBase
    {
        private bool on;
        public bool Tracking;
        private LineRenderer line;
        public Color LaserColor;
        private AudioSource Source;
        private float usetimer;
        public bool UsingHead;
        public bool ApplyForceToSelf = true;
        public Vector2 Offset;
        public SpriteRenderer GlowSprite;
        private List<PersonBehaviour> People = new List<PersonBehaviour>();
        public bool Behind;
        private Vector2 CurrentPos;
        private bool OmegaOn;
        private GameObject SoundPlayer;
        public AudioClip[] ShootSfx;

        public override void Start()
        {
            PowerName = "Laser Blaster";
            Activation = (UsingHead)? "Head" : "Hand";
            this.line = this.gameObject.AddComponent<LineRenderer>();
            line.startColor = this.LaserColor;
            line.endColor = this.LaserColor;
            line.startWidth = 0.05f;
            line.endWidth = 0.05f;
            line.material = UnityEngine.Object.Instantiate<Material>(ModAPI.FindMaterial("VeryBright"));
            line.material.SetColor("_GlowColour", new Color(LaserColor.r, LaserColor.g, LaserColor.b, 0.2f));
            line.material.SetFloat("_GlowIntensity", 0.2f);
            line.material.mainTexture = ModAPI.FindMaterial("Sprites-Default").mainTexture;
            line.sortingOrder = this.gameObject.GetComponent<SpriteRenderer>().sortingOrder + ((this.Behind) ? -1 : 1);
            line.sortingLayerID = this.gameObject.GetComponent<SpriteRenderer>().sortingLayerID;
            line.positionCount = 2;
            line.sortingLayerID = this.gameObject.GetComponent<SpriteRenderer>().sortingLayerID;
            line.enabled = false;
            this.Source = this.gameObject.AddComponent<AudioSource>();
            this.Source.loop = true;
            this.Source.clip = this.ShootSfx[0]; //Placeholder Sound
            this.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            this.SoundPlayer = new GameObject("Audio Player", new Type[] { typeof(AudioSource), typeof(AudioSourceTimeScaleBehaviour) });
            base.Start();
        }
        
        public override void UsePower() //I should've made the power use constant activation instead of toggle
        {
            if (Enabled && this.gameObject.GetComponent<LimbBehaviour>().Person.IsAlive())
            {
                if (this.Tracking)
                {
                    var HeadPos = this.transform.position + this.transform.TransformVector(this.Offset);
                    line.SetPosition(1, HeadPos + this.transform.TransformVector(new Vector2(1f, 0f) * this.transform.localScale.x));
                    Collider2D[] hits = Physics2D.OverlapCircleAll(this.transform.position, 50f, LayerMask.GetMask(new string[] { "Objects", "Bounds", "CollidingDebris" }));
                    foreach (var thing in hits)
                    {
                        if (thing.transform && thing.transform.root.gameObject != this.transform.root.gameObject && thing.transform.gameObject.tag == "Limb" && !People.Contains(thing.transform.gameObject.GetComponent<LimbBehaviour>().Person) && !this.OmegaOn)
                        {
                            line.enabled = true;
                            this.StartCoroutine(this.OmegaBeam(thing.transform.gameObject.GetComponent<LimbBehaviour>()));
                        }
                    }
                    return;
                }
                this.on = !this.on;
                if (this.on)
                {
                    line.enabled = true;
                    if (GlowSprite != null)
                    {
                        GlowSprite.enabled = true;
                    }
                    this.Source.PlayOneShot(this.ShootSfx[1], 2f); //Placeholder Sound
                    this.Source.Play();
                    if(ApplyForceToSelf) this.gameObject.GetComponent<Rigidbody2D>().AddForce((this.UsingHead)? -this.transform.right * this.transform.root.localScale.normalized.x : this.transform.up * 9f, ForceMode2D.Impulse);
                    return;
                }
                line.enabled = false;
                if (GlowSprite != null)
                {
                    GlowSprite.enabled = false;
                }
                this.Source.Stop();
                this.Source.PlayOneShot(this.ShootSfx[2], 1.2f); //Placeholder Sound
            }
        }

        private IEnumerator OmegaBeam(LimbBehaviour Limb)
        {
            this.OmegaOn = true;
            line.positionCount = 3;
            this.CurrentPos = (this.transform.position + this.transform.TransformVector(this.Offset)) + this.transform.TransformVector(new Vector2(1f, 0f) * this.transform.localScale.x);
            Vector2 LastPos = this.CurrentPos;
            bool Huck = true;
            int pos = 2;
            LimbBehaviour Limb2;
            var sound = SoundPlayer.GetComponent<AudioSource>();
            sound.clip = ResourceStorage.Sound_OmegaBeam;
            sound.loop = true;
            sound.Play();
            float time = 0f;
            float actualtime = 0f;
            while (Huck)
            {
                if (!this.gameObject.GetComponent<LimbBehaviour>().Person.IsAlive())
                {
                    sound.Stop();
                    line.enabled = false;
                    this.OmegaOn = false;
                    yield break;
                }
                if (Limb == null || Limb.gameObject == null)
                {
                    sound.Stop();
                    line.enabled = false;
                    this.OmegaOn = false;
                    yield break;
                }
                time += Time.deltaTime;
                actualtime += Time.deltaTime;
                if (time >= 0.2f)
                {
                    pos += 1;
                    line.positionCount += 1;
                    LastPos = this.CurrentPos;
                    time = 0f;
                }
                this.CurrentPos = Vector2.MoveTowards(this.CurrentPos, Limb.transform.position, Time.deltaTime * 25f);
                line.SetPosition(pos, this.CurrentPos);
                SoundPlayer.transform.position = this.CurrentPos;
                var cast = Physics2D.Raycast(this.CurrentPos, ((Vector2)Limb.transform.position - this.CurrentPos).normalized, 0.25f);
                if (cast.transform)
                {
                    if (cast.transform.root == this.transform.root && actualtime < 1f)
                    {
                        continue;
                    }
                    if (cast.transform.gameObject.TryGetComponent<LimbBehaviour>(out Limb2))
                    {
                        Limb2.Crush();
                    }
                    ExplosionCreator.CreateFragmentationExplosion(40U, this.CurrentPos, 4f, 6f, true, false, 0.05f);
                    Huck = false;
                }
                else if (Vector2.Distance(this.CurrentPos, Limb.transform.position) <= 0.075)
                {
                    Debug.Log("Test");
                    Limb.Crush();
                    ExplosionCreator.CreateFragmentationExplosion(40U, this.CurrentPos, 4f, 6f, true, false, 0.05f);
                    Huck = false;
                }
                yield return new WaitForEndOfFrame();
            }
            sound.Stop();
            line.enabled = false;
            this.OmegaOn = false;
            yield return new WaitForEndOfFrame();
        }

        private void OnDestroy()
        {
            SoundPlayer.GetComponent<AudioSource>().Stop();
            line.enabled = false;
            this.OmegaOn = false;
        }

        private void OnDisable()
        {
            SoundPlayer.GetComponent<AudioSource>().Stop();
            line.enabled = false;
            this.OmegaOn = false;
        }

        private void Update()
        {
            PhysicalBehaviour Phys;
            if (this.Enabled && this.Tracking)
            {
                var HeadPos = this.transform.position + this.transform.TransformVector(this.Offset);
                line.SetPosition(0, HeadPos);
            }
            if (this.Enabled && this.on)
            {
                line.SetPosition(0, this.transform.position + this.transform.TransformVector(this.Offset));
                RaycastHit2D hit = Physics2D.Raycast(this.transform.position, (this.UsingHead) ? this.transform.right * this.transform.root.localScale.normalized.x : -this.transform.up, 25000f, LayerMask.GetMask(new string[] { "Objects", "Bounds", "CollidingDebris" }));
                if (hit.transform && hit.transform.root.gameObject != this.transform.root.gameObject && hit.transform.gameObject.TryGetComponent<PhysicalBehaviour>(out Phys))
                {
                    Phys.Temperature = Mathf.Lerp(Phys.Temperature, 5000f, Time.deltaTime / 5f);
                    Phys.rigidbody.AddForceAtPosition((hit.normal * Time.timeScale) * -0.06f, hit.point, ForceMode2D.Impulse);
                    if (Phys.gameObject.GetComponent<LimbBehaviour>())
                    {
                        var Limb = Phys.gameObject.GetComponent<LimbBehaviour>();
                        Limb.Damage(2f * Time.timeScale);
                        Limb.Person.AddPain(1f * Time.timeScale);
                        Limb.SkinMaterialHandler.ImpactDamage(2f * Time.timeScale, hit.point);
                    }
                }
                line.SetPosition(1, hit.point);
                if (!this.gameObject.GetComponent<LimbBehaviour>().Person.IsAlive())
                {
                    line.enabled = false;
                    if (GlowSprite != null)
                    {
                        GlowSprite.enabled = false;
                    }
                    this.Source.Stop();
                    this.Source.PlayOneShot(this.ShootSfx[2], 1.2f);
                }
                // Cam, could you add impact particles for me? - Aspa
            }
        }

        /// <summary>
        /// Set up the power on a character.
        /// </summary>
        /// <param name="body">The selected limb of the user.</param>
        /// <param name="person">The PersonBehaviour of the user. (body.Person)</param>
        /// <param name="Instance">The overall instance of the user. ( body.Person.gameObject)</param>
        public static void SetUpPower(LimbBehaviour body, PersonBehaviour person, GameObject Instance, bool applyForceToSelf = true)
        {
            //This checks if the person already has the power, if they don't we give it to them (duplicates are bad) NOTE:make sure to change the class to your new power class
            // You can use GetOrAddComponent if you want here. - Aspa
            if (!body.GetComponent<LaserBlasterPower>())
            {
                //Adds your new power to the person NOTE:make sure to change the class to your new power class
                var e = body.gameObject.AddComponent<LaserBlasterPower>();
                e.ApplyForceToSelf = applyForceToSelf;
                //This sets the power's icon, make sure you add the your new icon to the ResourceStorage NOTE:make sure to change the class to your new power class
                body.GetComponent<LaserBlasterPower>().PowerIcon = ResourceStorage.Sprite_CyborgBlasterPowerIcon;

                body.gameObject.AddComponent<UseEventTrigger>().Action = () =>
                {
                    //This triggers your power NOTE:make sure to change the class to your new power class
                    body.gameObject.GetComponent<LaserBlasterPower>().UsePower();
                };
            }
        }
    }
}
