﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using System.Reflection;
namespace DCPP
{
    public class ShazamPower : PowerBase
    {
        public Texture2D Original;
        public Texture2D Altered;
        private bool UsingOriginal = false;
        public UnityAction OnAlter;
        public UnityAction OnOriginal;
        public AudioSource Aud;
        public AudioClip YellSound;
        public bool IsShazam = true;
        private float time = 1f;
        public string Name = "Gift of the Gods";

        public override void Start()
        {
            PowerName = this.Name;
            Activation = "Head";
            base.Start();
        }

        public void Update()
        {
            if (this.time < 1f)
            {
                this.time += Time.deltaTime;
            }
        }


        public override void UsePower()
        {
            if (Enabled && this.time >= 1f && this.GetComponent<LimbBehaviour>().Person.Consciousness > 0.8f)
            {
                this.time = 0f;
                this.Aud = this.gameObject.AddComponent<AudioSource>();
                this.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                if (this.IsShazam)
                {
                    GameObject gameObject = UnityEngine.Object.Instantiate<GameObject>(Resources.Load<GameObject>("Prefabs/LightningToolEntity"), this.transform.position + new Vector3(0f, 200f, 0f), Quaternion.identity);
                    var lightningEntity = gameObject.GetComponent<LightningToolEntityBehaviour>();
                    gameObject.SetActive(false);
                    lightningEntity.SingleBolt.enabled = false;
                    lightningEntity.LargeFlashLight.enabled = false;
                    lightningEntity.ParticleSystem.Play();
                    this.StartCoroutine(this.Bolt(lightningEntity));
                    return;
                }
                this.ChangeSkin();
            }
        }

        private IEnumerator Bolt(LightningToolEntityBehaviour Lightning)
        {
            this.Aud.PlayOneShot(this.YellSound, 2f);
            yield return new WaitForSeconds(0.5f);
            if (!this.GetComponent<LimbBehaviour>().Person.IsAlive())
            {
                yield break;
            }
            Lightning.SingleBolt.gameObject.SetActive(true);
            RaycastHit2D hit = Physics2D.Raycast(Lightning.transform.position, Vector2.down, 5000f, LayerMask.GetMask(new string[]{"Objects","Bounds"}));
            if (!hit)
            {
                yield break;
            }
            float x = UnityEngine.Random.value * 10000f;
            float num = hit.distance * 0.2f;
            var vertices = new Vector3[Lightning.SingleBolt.positionCount];
            this.Aud.PlayOneShot(Lightning.Thunder.PickRandom<AudioClip>());
            int num2 = vertices.Length;
            for (int j = 0; j < num2; j++)
            {
                float num3 = (float)j / (float)num2;
                float d = 1f - Mathf.Abs(2f * num3 - 1f);
                Vector3 a = (Utils.GetPerlin2Mapped(x, num3 / num * 19f) + (Vector3)UnityEngine.Random.insideUnitCircle * 0.2f) * d;
                vertices[j] = Vector3.Lerp(Lightning.transform.position, this.transform.position, num3) + num * a;
            }
            Lightning.SingleBolt.SetPositions(vertices);
            ModAPI.CreateParticleEffect("BigExplosion", this.transform.position);
            Lightning.SingleBolt.enabled = true;
            Lightning.LargeFlashLight.enabled = true;
            this.ChangeSkin();
            yield return new WaitForSeconds(0.064f);
            Lightning.SingleBolt.enabled = false;
            Lightning.LargeFlashLight.enabled = false;
            GameObject.Destroy(Lightning.gameObject);
            yield break;
        }

        public void ChangeSkin()
        {
            this.gameObject.GetComponent<LimbBehaviour>().Person.SetBodyTextures((this.UsingOriginal)? this.Original : this.Altered);
            if (this.UsingOriginal)
            {
                this.OnOriginal.Invoke();
            }
            else
            {
                this.OnAlter.Invoke();
            }
            this.UsingOriginal = !this.UsingOriginal;
        }

        public static void SetUpPower(LimbBehaviour body, Sprite Thumb)
        {
            if (!body.GetComponent<ShazamPower>())
            {
                body.gameObject.AddComponent<ShazamPower>();
                body.GetComponent<ShazamPower>().PowerIcon = Thumb;
                body.gameObject.AddComponent<UseEventTrigger>().Action = () =>
                {
                    body.gameObject.GetComponent<ShazamPower>().UsePower();
                };
            }
        }
    }
}
