using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace DCPP
{
    #region SyringeObjects

    internal class JokerLaughSyringe : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "JOKERPOISON";
        }
    }

    internal class LethalScreamSyringe : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "SCREAMPOISON";
        }
    }

    internal class LaserEyeSyringe : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "LASERPOISON";
        }
    }

    internal class OmegaBeamSyringe : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "OMEGABEAMPOISON";
        }
    }

    internal class TimeStopSyringeBehaviour : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "TIMESTOPPOISON";
        }
    }

    internal class BeetleSyringe : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "BEETLEPOISON";
        }
    }

    internal class VibrateSyringeBehaviour : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "VIBRATEPOISON";
        }
    }

    internal class MagicPushSyringeBehaviour : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "MAGICPUSHPOISON";
        }
    }

    internal class FlyingSyringeBehaviour : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "FLYINGPOISON";
        }
    }
    
    internal class FistProjectionBehaviour : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "FISTPROJECTIONPOISON";
        }
    }
    
    internal class BaneBoostBehaviour : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "BANEBOOSTPOISON";
        }
    }
    
    internal class FearBehaviour : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "FEARPOISON";
        }
    }

    internal class TelekinesisBehaviour : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "TELEKINESISPOISON";
        }
    }

    internal class ShockBehaviour : SyringeBehaviour
    {
        public override string GetLiquidID()
        {
            return "SHOCKPOISON";
        }
    }

    #endregion

    public class IvyPoison : Liquid
    {
        public IvyPoison()
        {
            this.Color = new Color32(129, 55, 231, 255);
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
        }

        public override void OnUpdate(BloodContainer container)
        {
            if (container.transform.root.gameObject.TryGetComponent<PersonBehaviour>(out PersonBehaviour person))
            {
                person.Wince(30f);
                person.OxygenLevel -= Time.deltaTime * 5f;
                if (UnityEngine.Random.value >= 0.95f)
                {
                    person.BrainDamaged = true;
                }
                if (UnityEngine.Random.value >= 0.99f)
                {
                    person.Heartbeat = 0f;
                }
            }
        }
    }

    public class JokerPoison : TemporaryBodyLiquid
    {
        public JokerPoison()
        {
            this.Color = Color.green; //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            PhysicalBehaviour physicalBehaviour = limb.PhysicalBehaviour;
            foreach (var body in physicalBehaviour.GetComponent<LimbBehaviour>().Person.Limbs)
            {
                if (body.gameObject.name.Contains("Head"))
                {
                    LaughPower.SetUpPower(body, physicalBehaviour.GetComponent<LimbBehaviour>().Person, physicalBehaviour.GetComponent<LimbBehaviour>().Person.gameObject);
                }
            }
            //Put funny code in here of what you want it to do.
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "JOKERPOISON";
    }

    public class laserPoison : TemporaryBodyLiquid
    {
        public laserPoison()
        {
            this.Color = Color.red; //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            PhysicalBehaviour physicalBehaviour = limb.PhysicalBehaviour;
            foreach (var body in limb.Person.Limbs)
            {
                if (body.gameObject.name.Contains("Head"))
                {
                    LaserBlasterPower.SetUpPower(body, physicalBehaviour.GetComponent<LimbBehaviour>().Person, physicalBehaviour.GetComponent<LimbBehaviour>().Person.gameObject);
                    var las = limb.Person.Limbs[0].gameObject.GetComponent<LaserBlasterPower>();
                    las.LaserColor = new Color(1f, 0f, 0f, 1f);
                    las.Offset = new Vector2(0.05f, 0.04f);
                    las.UsingHead = true; ///This is an example if you want the constant laser to appear at the head instead.
                    las.ShootSfx = new AudioClip[]
                    {
                        ResourceStorage.Sound_SLaserConstant,
                        ResourceStorage.Sound_LaserOn,
                        ResourceStorage.Sound_SLaserOff
                    };
                }
            }
            //Put funny code in here of what you want it to do.
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "LASERPOISON";
    }

    public class SpeedyPoison : TemporaryBodyLiquid
    {
        public SpeedyPoison()
        {
            this.Color = new Color32(150, 0, 0, 255); //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour Limb)
        {
            SpeedPower.SetUpPower(Limb.Person.Limbs[0], Limb.Person, Limb.gameObject);
            var speed = Limb.Person.Limbs[0].gameObject.GetComponent<SpeedPower>();
            speed.TrailColor1 = new Color(1f, 0f, 0f, 1f);
            speed.TrailColor2 = new Color(1f, 1f, 0f, 0f);
            //Put funny code in here of what you want it to do.
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "SPEEDFORCEPOISON";
    }

    public class TimeStopPoison : TemporaryBodyLiquid
    {
        public TimeStopPoison()
        {
            this.Color = new Color32(150, 0, 0, 255); //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour Limb)
        {
            TimeStop.SetUpPower(Limb.Person.Limbs[1], Limb.Person, Limb.gameObject);
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "TIMESTOPPOISON";
    }

    public class BeetlePoison : TemporaryBodyLiquid
    {
        public BeetlePoison()
        {
            this.Color = new Color32(0, 0, 150, 255); //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour Limb)
        {
            BeetleSuit.SetUpPower(Limb.Person.Limbs[1], Limb.Person, Limb.gameObject);
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "BEETLEPOISON";
    }

    public class SpeedsterVibratePoison : TemporaryBodyLiquid
    {
        public SpeedsterVibratePoison()
        {
            this.Color = new Color32(150, 0, 0, 255); //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour Limb)
        {
            SpeedsterVibrate.SetUpPower(Limb.Person.Limbs[11], Limb.Person, Limb.gameObject);
            SpeedsterVibrate.SetUpPower(Limb.Person.Limbs[13], Limb.Person, Limb.gameObject);
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "VIBRATEPOISON";
    }

    public class MagicPushPoison : TemporaryBodyLiquid
    {
        public MagicPushPoison()
        {
            this.Color = new Color32(240, 176, 0, 255); //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour Limb)
        {
            MagicPush.SetUpPower(Limb.Person.Limbs[11], Limb.Person, Limb.gameObject);
            MagicPush.SetUpPower(Limb.Person.Limbs[13], Limb.Person, Limb.gameObject);
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "MAGICPUSHPOISON";
    }

    public class FlyingPoison : TemporaryBodyLiquid
    {
        public FlyingPoison()
        {
            this.Color = new Color32(255, 255, 255, 255); //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour Limb)
        {
            Flying.SetUpPower(Limb.Person, Limb.gameObject);
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "FLYINGPOISON";
    }

    public class FistProjectionPoison : TemporaryBodyLiquid
    {
        public FistProjectionPoison()
        {
            this.Color = new Color32(13, 255, 0, 255); //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour Limb)
        {
            FistProjection.SetUpPower(Limb.Person.Limbs[11], Limb.Person, Limb.gameObject,Color.green);
            FistProjection.SetUpPower(Limb.Person.Limbs[13], Limb.Person, Limb.gameObject, Color.green);
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "FISTPROJECTIONPOISON";
    }

    public class BaneBoostPoison : TemporaryBodyLiquid
    {
        public BaneBoostPoison()
        {
            this.Color = new Color32(60, 145, 55, 255); //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour Limb)
        {
            BaneBoost.SetUpPower(Limb.Person.Limbs[0], Limb.Person, Limb.gameObject, false);
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "BANEBOOSTPOISON";
    }

    public class LethalScreamPoison : TemporaryBodyLiquid
    {
        public LethalScreamPoison()
        {
            this.Color = Color.cyan; //Color of the liquid
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            PhysicalBehaviour physicalBehaviour = limb.PhysicalBehaviour;
            foreach (var body in physicalBehaviour.GetComponent<LimbBehaviour>().Person.Limbs)
            {
                if (body.gameObject.name.Contains("Head"))
                {
                    HeadExploderPower.SetUpPower(body, physicalBehaviour.GetComponent<LimbBehaviour>().Person, physicalBehaviour.GetComponent<LimbBehaviour>().Person.gameObject);
                }
            }
            //Put funny code in here of what you want it to do.
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "SCREAMPOISON";
    }

    public class OmegaBeamPoison : TemporaryBodyLiquid
    {
        public OmegaBeamPoison()
        {
            this.Color = new Color(0.8f, 0.3f, 0f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            var person = limb.Person;
            LaserBlasterPower.SetUpPower(person.Limbs[0], person, person.gameObject);
            var las = limb.Person.Limbs[0].GetComponent<LaserBlasterPower>();
            las.Tracking = true;
            las.LaserColor = new Color(1f, 0f, 0f, 1f);
            las.Offset = new Vector2(0.05f, 0.04f);
            las.ShootSfx = new AudioClip[] //this literally does nothing, but I'm too lazy to make it so tracking mode doesn't check for audio
            {
                ResourceStorage.Sound_SLaserConstant,
                ResourceStorage.Sound_LaserOn,
                ResourceStorage.Sound_SLaserOff
            };
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "OMEGABEAMPOISON";
    }

    public class FearPoison : TemporaryBodyLiquid
    {
        
        public FearPoison()
        {
            this.Color = new Color(0.8f, 0.3f, 0f);
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.StartCoroutine(FearPoisonEffect(limb));
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "FEARPOISON";

        IEnumerator FearPoisonEffect(LimbBehaviour limb)
        {
            limb.Person.Wince(20);
            limb.Person.AddPain(20);
            yield return new WaitForSeconds(2f);
            limb.Person.Wince(20);
            limb.Person.AddPain(20);
            yield return new WaitForSeconds(2f);
            limb.Person.Wince(20);
            limb.Person.AddPain(20);
            yield return new WaitForSeconds(2f);
            limb.Person.Wince(20);
            limb.Person.AddPain(20);
            yield return new WaitForSeconds(2f);
            limb.Person.Wince(20);
            limb.Person.AddPain(20);
            yield return new WaitForSeconds(2f);
            limb.Person.Wince(20);
            limb.Person.AddPain(20);
            yield return new WaitForSeconds(2f);
            limb.Person.Wince(20);
            limb.Person.AddPain(20);
            yield return new WaitForSeconds(2f);
            
        }
    }

    public class TelekinesisPoison : TemporaryBodyLiquid
    {

        public TelekinesisPoison()
        {
            this.Color = Color.blue;
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }
        
        public override void OnEnterLimb(LimbBehaviour limb)
        {
            Telekinesis.SetUpPower(limb.Person.Limbs[11], limb.Person, null);
            Telekinesis.SetUpPower(limb.Person.Limbs[13], limb.Person, null);
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "TELEKINESISPOISON";
    }

    public class ShockPoison : TemporaryBodyLiquid
    {

        public ShockPoison()
        {
            this.Color = Color.white;
        }

        public override void OnEnterContainer(BloodContainer container)
        {
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            ElectroShock.SetUpPower(limb.Person.Limbs[11], limb.Person,limb.Person.gameObject);
            ElectroShock.SetUpPower(limb.Person.Limbs[13], limb.Person, limb.Person.gameObject);
        }

        public override void OnExitContainer(BloodContainer container)
        {
        }

        public const string ID = "SHOCKPOISON";
    }


}    
